package jalview.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.ContainerEvent;
import java.awt.event.ContainerListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.event.WindowStateListener;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.TitledBorder;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.table.*;

import compbio.metadata.Argument;
import compbio.metadata.Option;
import compbio.metadata.Parameter;
import compbio.metadata.Preset;
import compbio.metadata.PresetManager;
import compbio.metadata.RunnerConfig;
import compbio.metadata.ValueConstrain;
import compbio.metadata.WrongParameterException;
import compbio.metadata.ValueConstrain.Type;

import jalview.ws.jws2.Jws2Discoverer;
import jalview.ws.jws2.Jws2Discoverer.Jws2Instance;

/**
 * job parameter editing/browsing dialog box. User can browse existing settings
 * (user + presets + Defaults), and any changes to parameters creates a modified
 * user parameter set. LOGIC: If the parameter set is modified, and its name is
 * a valid, non-existant user parameter set, then a save button is shown. If the
 * parameter set is modified and its name is a valid, extant user parameter set,
 * then an update button is shown. If user parameter set's name is edited, and
 * old name exists as a writable user parameter set, then rename button is
 * shown. If current parameter set is associated with a user defined parameter
 * set, then : if set is modifed, a 'revert' button is shown. if set is not
 * modified, a 'delete' button is shown.
 * 
 * @author JimP
 * 
 */
public class WsJobParameters extends JPanel implements ItemListener,
        ActionListener, DocumentListener
{
  private static final String SVC_DEF = "Defaults"; // this is the null
                                                    // parameter set as shown to
                                                    // user

  private static final int PARAM_WIDTH = 340, PARAM_HEIGHT = 150,
          PARAM_CLOSEDHEIGHT = 80;

  private static final int OPTSET_HEIGHT = 30;

  JPanel SetNamePanel = new JPanel();

  JPanel setDetails = new JPanel();

  JSplitPane settingsPanel = new JSplitPane();

  JSplitPane jobPanel = new JSplitPane();

  JPanel jobOptions = new JPanel();

  JScrollPane jobOptionsPane = new JScrollPane();

  JPanel jobParameters = new JPanel();

  JButton savmodified = new JButton();

  JButton renmodified = new JButton();

  JButton deletesetting = new JButton();

  JButton revertsetting = new JButton();

  JButton startjob = new JButton();

  JButton canceljob = new JButton();

  JComboBox setName = new JComboBox();

  JTextArea setDescr = new JTextArea();

  JScrollPane paramPane = new JScrollPane();

  // JList paramList = new JList();
  JPanel paramList = new JPanel();

  RunnerConfig serviceOptions;

  private BorderLayout jparamLayout;

  WsJobParameters(Jws2Instance service)
  {
    this(service, null);
  }

  public WsJobParameters(Jws2Instance service, Preset p)
  {
    this(null, service, p, null);
  }

  /**
   * 
   * @param desktop
   *          - if null, create new JFrame outside of desktop
   * @param service
   * @param p
   */
  public WsJobParameters(JFrame parent, Jws2Instance service, Preset p,
          List<Argument> jobArgset)
  {
    super();
    jbInit();
    // argSetModified(false);
    // populate parameter table
    initForService(service, p, jobArgset);
    // display in new JFrame attached to parent.
    validate();
  }

  int response = -1;

  JDialog frame = null;

  public boolean showRunDialog()
  {

    frame = new JDialog(Desktop.instance, true);

    frame.setTitle("Edit parameters for " + service.getActionText());
    Rectangle deskr = Desktop.instance.getBounds();
    frame.setBounds(new Rectangle((int) (deskr.getCenterX() - 240),
            (int) (deskr.getCenterY() - 92), 380, 385));
    frame.setContentPane(this);

    frame.setVisible(true);

    if (response > 0)
    {
      return true;
    }
    return false;
  }

  protected JButton makeButton(String label, String tooltip,
          ActionListener action)
  {
    JButton button = new JButton();
    button.setText(label);
    button.setFont(new java.awt.Font("Verdana", Font.PLAIN, 10));
    button.setForeground(Color.black);
    button.setHorizontalAlignment(SwingConstants.CENTER);
    button.setToolTipText(tooltip);
    button.addActionListener(action);
    return button;
  }

  private void jbInit()
  {
    savmodified = makeButton("Save", "Not implemented yet :) ",
            new ActionListener()
            {

              public void actionPerformed(ActionEvent e)
              {
                savModified_actionPerformed(e);
              }
            });
    renmodified = makeButton("Rename", "Not implemented yet :) ",
            new ActionListener()
            {

              public void actionPerformed(ActionEvent e)
              {
                renModified_actionPerformed(e);
              }
            });
    deletesetting = makeButton("Delete", "Not implemented yet :) ",
            new ActionListener()
            {

              public void actionPerformed(ActionEvent e)
              {
                deleteSetting_actionPerformed(e);
              }
            });
    revertsetting = makeButton("Revert", "Undo changes to parameters.",
            new ActionListener()
            {

              public void actionPerformed(ActionEvent e)
              {
                revertSetting_actionPerformed(e);
              }
            });

    startjob.setFont(new java.awt.Font("Verdana", Font.PLAIN, 10));
    startjob.setText("Start");
    startjob.setToolTipText("Start Job");
    startjob.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        startjob_actionPerformed(e);
      }
    });
    canceljob.setFont(new java.awt.Font("Verdana", Font.PLAIN, 10));
    canceljob.setText("Cancel");
    canceljob.setToolTipText("Cancel Job");
    canceljob.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        canceljob_actionPerformed(e);
      }
    });

    setDetails.setBorder(new TitledBorder("Details"));
    setDetails.setLayout(new BorderLayout());
    setDescr.setColumns(40);
    setDescr.setWrapStyleWord(true);
    setDescr.setLineWrap(true);
    setDescr.setBackground(getBackground());
    setDescr.setEditable(true);
    setDescr.getDocument().addDocumentListener(this);
    JScrollPane setDescrView = new JScrollPane();
    // setDescrView.setPreferredSize(new Dimension(350, 200));
    setDescrView.getViewport().setView(setDescr);
    setName.setEditable(true);
    setName.addItemListener(this);
    setName.getEditor().addActionListener(this);
    SetNamePanel.setLayout(new BorderLayout());
    SetNamePanel.add(setName, BorderLayout.WEST);
    // initial button visibility
    deletesetting.setVisible(false);
    revertsetting.setVisible(false);
    renmodified.setVisible(false);
    savmodified.setVisible(false);
    JPanel setsavebuts = new JPanel();
    setsavebuts.setLayout(new FlowLayout());
    setsavebuts.add(revertsetting, BorderLayout.CENTER);
    setsavebuts.add(renmodified, BorderLayout.CENTER);
    setsavebuts.add(deletesetting, BorderLayout.CENTER);
    setsavebuts.add(savmodified, BorderLayout.EAST);
    SetNamePanel.add(setsavebuts,BorderLayout.EAST);
    setDetails.add(setDescrView, BorderLayout.CENTER);
    // setDetails.setPreferredSize(new Dimension(360, 100));
    jobParameters.setBorder(new TitledBorder("Parameters"));
    jobParameters.setLayout(jparamLayout = new BorderLayout());
    paramPane.setPreferredSize(new Dimension(360, 300));
    paramPane.getVerticalScrollBar().setUnitIncrement(20);
    // paramPanel.setPreferredSize(new Dimension(360, 300));
    // TODO: relayout buttons nicely
    paramPane.getViewport().setView(paramList);
    jobParameters.add(paramPane, BorderLayout.CENTER);
    JPanel jobOptionsPanel = new JPanel();
    jobOptionsPanel.setLayout(new BorderLayout());
    jobOptionsPanel.setBorder(new TitledBorder("Options"));
    jobOptionsPane.getViewport().setView(jobOptions);
    jobOptionsPanel.add(jobOptionsPane, BorderLayout.CENTER);
    settingsPanel.setLeftComponent(jobOptionsPanel);
    settingsPanel.setRightComponent(jobParameters);
    settingsPanel.setOrientation(JSplitPane.VERTICAL_SPLIT);
    settingsPanel.setDividerLocation(0.4);

    setLayout(new BorderLayout());
    // setPreferredSize(new Dimension(400, 600));
    // setSize(new Dimension(400, 600));
    jobPanel.setLeftComponent(setDetails);
    jobPanel.setRightComponent(settingsPanel);
    jobPanel.setOrientation(JSplitPane.VERTICAL_SPLIT);
    jobPanel.setDividerLocation(0.8);
    add(SetNamePanel, BorderLayout.NORTH);
    add(jobPanel, BorderLayout.CENTER);
    JPanel dialogpanel = new JPanel();
    dialogpanel.add(startjob);
    dialogpanel.add(canceljob);
    add(dialogpanel, BorderLayout.SOUTH);
  }

  protected void revertSetting_actionPerformed(ActionEvent e)
  {
    // TODO Auto-generated method stub

  }

  protected void deleteSetting_actionPerformed(ActionEvent e)
  {
    String setname = (String) setName.getSelectedItem();
    int p=setName.getSelectedIndex();
    if (_getUserPreset(setname)!=null)
    {
      _deleteUserPreset(setname);
      
    }
    if (p>0 && p+1==setName.getItemCount())
    {
      p--;
    }
    setName.setSelectedIndex(p);
    
  }

  protected void renModified_actionPerformed(ActionEvent e)
  {
    if (curSetName==null || _getUserPreset(lastSetName)==null) {
      System.err.println("can't rename - names unchanged or original name not a preset.");
      return;
    }
    _deleteUserPreset(lastSetName);
    lastSetName=curSetName;
    savModified_actionPerformed(e);
    curSetName=null;
    boolean setd = settingDialog;
    settingDialog=true;
    syncSetNamesWithStore();
    settingDialog=setd;
  }

  protected void savModified_actionPerformed(ActionEvent e)
  {
    _storeUserPreset(lastSetName=(String) setName.getSelectedItem(), setDescr.getText(), getJobParams());
    curSetName=null;
    initArgSetModified(); // reset the modification state
  }

  protected void canceljob_actionPerformed(ActionEvent e)
  {
    response = 0;
    if (frame != null)
    {
      frame.setVisible(false);
    }
  }

  protected void startjob_actionPerformed(ActionEvent e)
  {
    response = 1;
    if (frame != null)
    {
      frame.setVisible(false);
    }
  }

  Jws2Instance service;

  /**
   * list of service presets in the gui
   */
  Hashtable servicePresets = null;

  /**
   * set if dialog is being set - so handlers will avoid spurious events
   */
  boolean settingDialog = false;

  void initForService(Jws2Instance service, Preset p,
          List<Argument> jobArgset)
  {
    settingDialog = true;
    this.service = service;
    // TODO: Recover window geometry prefs for this service
    // jobPanel.setDividerLocation(proportionalLocation)
    // settingsPanel.setDividerLocation(proportionalLocation)
    Hashtable exnames = new Hashtable();
    for (int i = 0, iSize = setName.getItemCount(); i < iSize; i++)
    {
      exnames.put((String) setName.getItemAt(i), setName.getItemAt(i));
    }
    // Add the default entry - if not present already.
    if (!exnames.contains(SVC_DEF))
    {
      setName.addItem(SVC_DEF);
      exnames.put(SVC_DEF, SVC_DEF);
    }
    serviceOptions = service.getRunnerConfig();
    // add any presets not already added.
    String curname = (p == null ? "" : p.getName());
    PresetManager prman = service.getPresets();
    servicePresets = new Hashtable();
    if (prman != null)
    {
      List prList = service.getPresets().getPresets();
      if (prList != null)
      {
        for (Object pr : prList)
        {
          servicePresets.put(((Preset) pr).getName(), "preset");
          if (!exnames.contains(((Preset) pr).getName()))
          {
            setName.addItem(((Preset) pr).getName());
          }
        }
      }
    }
    updateTable(p, jobArgset);
    initArgSetModified();
    settingDialog = false;

  }

  @SuppressWarnings("unchecked")
  private void updateTable(Preset p, List<Argument> jobArgset)
  {
    List<Parameter> setargs = new ArrayList<Parameter>();
    // populate table from default parameter set.
    List<Argument> args = serviceOptions.getArguments();

    // split to params and required arguments
    {
      for (Argument arg : args)
      {
        Argument myarg = (Argument) arg;
        // Ideally, Argument would implement isRequired !
        if (myarg instanceof Parameter)
        {
          Parameter parm = (Parameter) myarg;
          addParameter(parm);
        }
        else
        {
          if (myarg instanceof Option)
          {
            Option opt = (Option) myarg;
            addOption(opt).resetToDefault();
          }
          else
          {
            System.err.println("Ignoring unknown service argument type "
                    + arg.getClass().getName());
          }
        }
      }
      args = null; // no more args to process.
    }
    if (p != null)
    {
      // initialise setname
      setName.setSelectedItem(lastSetName = p.getName());
      setDescr.setText(lastDescrText = p.getDescription());
      // TODO - URL link
      try
      {
        args = p.getArguments(serviceOptions);
      } catch (Exception e)
      {
        e.printStackTrace();
      }
      // TODO: check if args should be unselected prior to resetting using the
      // preset
      setargs.clear();
    }
    else
    {
      if (lastParmSet == null)
      {
        // first call - so create a dummy name
        setName.setSelectedItem(lastSetName = SVC_DEF);
      }
    }

    if (jobArgset != null)
    {
      argSetModified(jobArgset, true);
      args = jobArgset;
    }
    // get setargs from current object
    if (args != null)
    {
      for (Argument arg : args)
      {
        if (arg instanceof Parameter)
        {
          setParameter((Parameter) arg);
        }
        else
        {
          if (arg instanceof Option)
          {
            System.out.println("Setting option " + arg.getName() + " with "
                    + arg.getDefaultValue());
            selectOption((Option) arg, arg.getDefaultValue());
          }
        }

      }
    }

    jobOptions.setPreferredSize(new Dimension(PARAM_WIDTH, optSet.size()
            * OPTSET_HEIGHT));
    jobOptions.setLayout(new GridLayout(optSet.size(), 1));
    refreshParamLayout();
    paramPane.validate();
    validate();
  }

  private boolean isModified()
  {
    return modifiedElements.size() > 0;
  }

  private Hashtable modifiedElements = new Hashtable();

  /**
   * reset gui and modification state settings
   */
  private void initArgSetModified()
  {
    curSetName = null;
    modifiedElements.clear();
    renmodified.setVisible(false);
    savmodified.setVisible(false);

  }

  private void argSetModified(Object modifiedElement, boolean b)
  {
    if (settingDialog)
    {
      return;
    }

    if (!b)
    {
      modifiedElements.remove(modifiedElement);
    }
    else
    {
      modifiedElements.put(modifiedElement, modifiedElement);
    }
    // set mod status based on presence of elements in table
    if (modifiedElements.size() > 0)
    {
      makeSetNameValid();
      savmodified.setVisible(true);
      revertsetting.setVisible(false);
    }
    else
    {
      revertsetting.setVisible(false);
      deletesetting
              .setVisible(!isServicePreset((String) setName
                      .getSelectedItem())
                      && _getUserPreset((String) setName.getSelectedItem()) != null);
      savmodified.setVisible(false);
    }
    // special reveal if setName has been modified
    if (modifiedElements.get(setName) != null)
    {
      if (curSetName != null && lastSetName != null
              && !lastSetName.equals(curSetName))
      {
        renmodified.setVisible(!isServicePreset(lastSetName));
      }
    }
    else
    {
      // setname isn't in modlist - so don't rename
      renmodified.setVisible(false);
    }
    validate();
  }

  private boolean isServicePreset(String selectedItem)
  {
    return selectedItem.equals(SVC_DEF)
            || servicePresets.containsKey(selectedItem);
  }

  /**
   * check if the current set name is a valid set name for saving, if not, then
   * fix it.
   */
  private void makeSetNameValid()
  {
    boolean stn = settingDialog;
    boolean renamed = false;
    settingDialog = true;
    String nm = (String) setName.getSelectedItem();
    // check if the name is reserved - if it is, rename it.
    if (isServicePreset(nm))
    {
      nm = "User " + nm;
      renamed = true;
    }
    // if ()
    // if nm exists in user's preset store then savmodified will update an
    // existing user defined preset
    // if nm doesn't exist, then the button will create a new preset.

    boolean makeupdate = false;
    // sync the gui with the preset database
    for (int i = 0, iS = setName.getItemCount(); i < iS; i++)
    {
      String snm = (String) setName.getItemAt(i);
      if (snm.equals(nm))
      {
        makeupdate = true;
        setName.setSelectedIndex(i);
      }
    }

    if (_getUserPreset(nm) != null)
    {
      savmodified.setText("Update");
    }
    else
    {
      if (renamed)
      {
        setName.addItem(nm);
        setName.setSelectedIndex(setName.getItemCount() - 1);
      }
      savmodified.setText("Save");
    }
    settingDialog = stn;
  }

  private void addParameter(Parameter parm)
  {
    ParamBox pb = paramSet.get(parm.getName());
    if (pb == null)
    {
      pb = new ParamBox(this, parm);
      paramSet.put(parm.getName(), pb);
      paramList.add(pb);
    }
    pb.init();
    // take the defaults from the parameter
    pb.updateControls(parm);
  }

  private void setParameter(Parameter arg)
  {
    ParamBox pb = paramSet.get(arg.getName());
    if (pb == null)
    {
      addParameter(arg);
    }
    else
    {
      pb.updateControls(arg);
    }

  }

  private void selectOption(Option opt, String string)
  {
    OptionBox cb = optSet.get(opt.getName());
    if (cb == null)
    {
      cb = addOption(opt);
    }
    cb.enabled.setSelected(true); // initial state for an option.
    if (string != null)
    {
      if (opt.getPossibleValues().contains(string))
      {
        cb.val.setSelectedItem(string);
      }
      else
      {
        throw new Error("Invalid value " + string + " for option " + opt);
      }

    }
    if (opt.isRequired() && !cb.enabled.isSelected())
    {
      // TODO: indicate paramset is not valid.. option needs to be selected!
    }
    cb.setInitialValue();
  }

  Map<String, ParamBox> paramSet = new Hashtable<String, ParamBox>();

  public class ParamBox extends JPanel implements ChangeListener,
          ActionListener
  {
    JButton showDesc = new JButton();

    JTextArea string = new JTextArea();

    JScrollPane descPanel = new JScrollPane();

    JSlider slider = null;

    JTextField valueField = null;

    ValueConstrain validator = null;

    JPanel settingPanel = new JPanel();

    JPanel controlPanel = new JPanel();

    boolean integ = false;

    boolean choice = false;

    boolean descisvisible = false;

    final WsJobParameters pmdialogbox;

    public ParamBox(final WsJobParameters pmlayout, Parameter parm)
    {
      pmdialogbox = pmlayout;
      setPreferredSize(new Dimension(PARAM_WIDTH, PARAM_CLOSEDHEIGHT));
      setBorder(new TitledBorder(parm.getName()));
      setLayout(null);
      showDesc.setFont(new Font("Verdana", Font.PLAIN, 6));
      showDesc.setText("+");
      string.setFont(new Font("Verdana", Font.PLAIN, 11));
      string.setBackground(getBackground());
      // string.setSize(new Dimension(PARAM_WIDTH, 80));
      string.setEditable(false);
      descPanel.getViewport().setView(string);
      // descPanel.setLocation(2,17);
      descPanel.setVisible(false);
      // string.setMinimumSize(new Dimension(140,80));
      // string.setMaximumSize(new Dimension(280,80));
      final ParamBox me = this;
      showDesc.addActionListener(new ActionListener()
      {

        @Override
        public void actionPerformed(ActionEvent e)
        {
          descisvisible = !descisvisible;
          descPanel.setVisible(descisvisible);
          me.setPreferredSize(new Dimension(PARAM_WIDTH,
                  (descisvisible) ? PARAM_HEIGHT : PARAM_CLOSEDHEIGHT));
          me.validate();
          pmlayout.refreshParamLayout();
        }
      });
      string.setWrapStyleWord(true);
      string.setLineWrap(true);
      string.setColumns(32);
      string.setText(parm.getDescription());
      JPanel firstrow = new JPanel();
      firstrow.setLayout(null);
      controlPanel.setLayout(new BorderLayout());
      controlPanel.setBounds(new Rectangle(39, 10, PARAM_WIDTH - 70,
              PARAM_CLOSEDHEIGHT - 50));
      showDesc.setBounds(new Rectangle(10, 10, 16, 16));
      firstrow.add(showDesc);
      firstrow.add(controlPanel);
      firstrow.setBounds(new Rectangle(10, 20, PARAM_WIDTH - 30,
              PARAM_CLOSEDHEIGHT - 30));
      add(firstrow);
      validator = parm.getValidValue();
      parameter = parm;
      if (validator != null)
      {
        integ = validator.getType() == Type.Integer;
      }
      else
      {
        if (parameter.getPossibleValues() != null)
        {
          choice = true;
        }
      }
      updateControls(parm);
      descPanel.setBounds(new Rectangle(10, PARAM_CLOSEDHEIGHT,
              PARAM_WIDTH - 20, PARAM_HEIGHT - PARAM_CLOSEDHEIGHT - 5));
      add(descPanel);
      validate();
    }

    public void init()
    {
      // reset the widget's initial value.
      lastVal = null;
    }

    boolean adjusting = false;

    Parameter parameter;

    JComboBox choicebox;

    public int getBoxHeight()
    {
      return (descisvisible ? PARAM_HEIGHT : PARAM_CLOSEDHEIGHT);
    }

    public void updateControls(Parameter parm)
    {
      adjusting = true;
      boolean init = (choicebox == null && valueField == null);
      float fVal = 0f;
      int iVal = 0;
      if (init)
      {
        if (choice)
        {
          choicebox = new JComboBox();
          choicebox.addActionListener(this);
          controlPanel.add(choicebox, BorderLayout.CENTER);
        }
        else
        {
          slider = new JSlider();
          slider.addChangeListener(this);
          valueField = new JTextField();
          valueField.addActionListener(this);
          valueField.setPreferredSize(new Dimension(60, 25));
          controlPanel.add(slider, BorderLayout.WEST);
          controlPanel.add(valueField, BorderLayout.EAST);

        }
      }

      if (parm != null)
      {
        if (choice)
        {
          if (init)
          {
            List vals = parm.getPossibleValues();
            for (Object val : vals)
            {
              choicebox.addItem(val);
            }
          }

          if (parm.getDefaultValue() != null)
          {
            choicebox.setSelectedItem(parm.getDefaultValue());
          }
        }
        else
        {
          valueField.setText(parm.getDefaultValue());
        }
      }
      lastVal = updateSliderFromValueField();
      adjusting = false;
    }

    Object lastVal;

    public Parameter getParameter()
    {
      try
      {
        if (choice)
        {
          parameter.setDefaultValue((String) choicebox.getSelectedItem());
        }
        else
        {
          parameter.setDefaultValue(valueField.getText());
        }
      } catch (WrongParameterException e)
      {
        e.printStackTrace();
        return null;
      }
      return parameter;
    }

    public Object updateSliderFromValueField()
    {
      int iVal;
      float fVal;
      if (validator != null)
      {
        if (integ)
        {
          iVal = 0;
          try
          {
            valueField.setText(valueField.getText().trim());
            iVal = Integer.valueOf(valueField.getText());
          } catch (Exception e)
          {
          }
          ;
          if (validator.getMin() != null && validator.getMax() != null)
          {
            slider.getModel().setRangeProperties(iVal, 1,
                    validator.getMin().intValue(),
                    validator.getMax().intValue(), true);
          }
          else
          {
            slider.setVisible(false);
          }
          return new int[]
          { iVal };
        }
        else
        {
          fVal = 0f;
          try
          {
            fVal = Float.valueOf(valueField.getText());
          } catch (Exception e)
          {
          }
          ;
          if (validator.getMin() != null && validator.getMax() != null)
          {
            slider.getModel().setRangeProperties((int) fVal * 1000, 1,
                    (int) validator.getMin().floatValue() * 1000,
                    (int) validator.getMax().floatValue() * 1000, true);
          }
          else
          {
            slider.setVisible(false);
          }
          return new float[]
          { fVal };
        }
      }
      else
      {
        if (!choice)
        {
          slider.setVisible(false);
          return new String[]
          { valueField.getText().trim() };
        }
        else
        {
          return new String[]
          { (String) choicebox.getSelectedItem() };
        }
      }

    }

    @Override
    public void stateChanged(ChangeEvent e)
    {
      if (!adjusting)
      {
        valueField.setText(""
                + ((integ) ? ("" + (int) slider.getValue())
                        : ("" + (float) (slider.getValue() / 1000f))));
        checkIfModified();
      }

    }

    @Override
    public void actionPerformed(ActionEvent e)
    {
      if (adjusting)
      {
        return;
      }
      if (!choice)
      {
        updateSliderFromValueField();
      }
      checkIfModified();
    }

    private void checkIfModified()
    {
      Object cstate = updateSliderFromValueField();
      boolean notmod = false;
      if (cstate.getClass() == lastVal.getClass())
      {
        if (cstate instanceof int[])
        {
          notmod = (((int[]) cstate)[0] == ((int[]) lastVal)[0]);
        }
        else if (cstate instanceof float[])
        {
          notmod = (((float[]) cstate)[0] == ((float[]) lastVal)[0]);
        }
        else if (cstate instanceof String[])
        {
          notmod = (((String[]) cstate)[0].equals(((String[]) lastVal)[0]));
        }
      }
      pmdialogbox.argSetModified(this, !notmod);
    }
  }

  Map<String, OptionBox> optSet = new Hashtable<String, OptionBox>();

  public class OptionBox extends JPanel implements ActionListener
  {
    JComboBox val = new JComboBox();

    JCheckBox enabled = new JCheckBox();

    Option option;

    public OptionBox(Option opt)
    {
      option = opt;
      setLayout(new BorderLayout());
      enabled.setSelected(opt.isRequired()); // TODO: lock required options
      enabled.setFont(new Font("Verdana", Font.PLAIN, 11));
      enabled.setText(opt.getName());
      enabled.setToolTipText(opt.getDescription());
      enabled.addActionListener(this);
      add(enabled, BorderLayout.NORTH);
      if (opt.getPossibleValues().size() > 1)
      {
        setLayout(new GridLayout(1, 2));
        for (Object str : opt.getPossibleValues())
        {
          val.addItem((String) str);
        }
        val.setSelectedItem((String) opt.getDefaultValue());
        val.addActionListener(this);
        add(val, BorderLayout.SOUTH);
      }
      // TODO: add actionListeners for popup (to open further info),
      // and to update list of parameters if an option is enabled
      // that takes a value.
      setInitialValue();
    }

    public void resetToDefault()
    {
      enabled.setSelected(false);
      if (option.isRequired())
      {
        // Apply default value
        selectOption(option, option.getDefaultValue());
      }
    }

    boolean initEnabled = false;

    String initVal = null;

    public void setInitialValue()
    {
      initEnabled = enabled.isSelected();
      if (option.getPossibleValues() != null
              && option.getPossibleValues().size() > 1)
      {
        initVal = (String) val.getSelectedItem();
      }
      else
      {
        initVal = null;
      }
    }

    public Option getOptionIfEnabled()
    {
      if (!enabled.isSelected())
      {
        return null;
      }
      try
      {
        if (val.getSelectedItem() != null)
        {
          option.setDefaultValue((String) val.getSelectedItem());
        }
      } catch (WrongParameterException e)
      {
        e.printStackTrace();
        return null;
      }
      return option;
    }

    @Override
    public void actionPerformed(ActionEvent e)
    {
      if (e.getSource() != enabled)
      {
        enabled.setSelected(true);
      }
      checkIfModified();
    }

    private void checkIfModified()
    {
      boolean notmod = (initEnabled == enabled.isSelected());
      if (enabled.isSelected() && initVal != null)
      {
        notmod |= initVal.equals(val.getSelectedItem());
      }
      argSetModified(this, !notmod);
    }
  }

  private OptionBox addOption(Option opt)
  {
    OptionBox cb = optSet.get(opt.getName());
    if (cb == null)
    {
      cb = new OptionBox(opt);
      optSet.put(opt.getName(), cb);
      jobOptions.add(cb);
    }
    return cb;
  }

  protected void refreshParamLayout()
  {
    int s = 100;
    for (ParamBox pbox : paramSet.values())
    {
      s += pbox.getBoxHeight();
    }
    paramList.setPreferredSize(new Dimension(PARAM_WIDTH, s));
    paramList.setLayout(new FlowLayout());
    validate();
  }

  /**
   * testing method - grab a service and parameter set and show the window
   * 
   * @param args
   */
  public static void main(String[] args)
  {
    jalview.ws.jws2.Jws2Discoverer disc = jalview.ws.jws2.Jws2Discoverer
            .getDiscoverer();
    int p = 0;
    if (args.length > 3)
    {
      Vector<String> services = new Vector<String>();
      services.addElement(args[p++]);
      Jws2Discoverer.setServiceUrls(services);
    }
    try
    {
      disc.run();
    } catch (Exception e)
    {
      System.err.println("Aborting. Problem discovering services.");
      e.printStackTrace();
      return;
    }
    Jws2Discoverer.Jws2Instance lastserv = null;
    for (Jws2Discoverer.Jws2Instance service : disc.getServices())
    {
      lastserv = service;
      if (p < args.length && service.serviceType.equalsIgnoreCase(args[p]))
      {
        break;
      }
    }
    if (lastserv != null)
    {
      List<Preset> prl = null;
      Preset pr = null;
      if (++p < args.length)
      {
        PresetManager prman = lastserv.getPresets();
        if (prman != null)
        {
          pr = prman.getPresetByName(args[p]);
          if (pr == null)
          {
            // just grab the last preset.
            prl = prman.getPresets();
          }
        }
      }
      Iterator<Preset> en = (prl == null) ? null : prl.iterator();
      while (true)
      {
        if (en != null)
        {
          if (!en.hasNext())
          {
            en = prl.iterator();
          }
          pr = en.next();
        }
        WsJobParameters pgui = new WsJobParameters(lastserv, pr);
        JFrame jf = new JFrame("Parameters for " + lastserv.getActionText());
        JPanel cont = new JPanel();
        // jf.setPreferredSize(new Dimension(600, 800));
        cont.add(pgui);
        jf.add(cont);
        final Thread thr = Thread.currentThread();
        jf.addWindowListener(new WindowListener()
        {

          @Override
          public void windowActivated(WindowEvent e)
          {
            // TODO Auto-generated method stub

          }

          @Override
          public void windowClosed(WindowEvent e)
          {
          }

          @Override
          public void windowClosing(WindowEvent e)
          {
            thr.interrupt();

          }

          @Override
          public void windowDeactivated(WindowEvent e)
          {
            // TODO Auto-generated method stub

          }

          @Override
          public void windowDeiconified(WindowEvent e)
          {
            // TODO Auto-generated method stub

          }

          @Override
          public void windowIconified(WindowEvent e)
          {
            // TODO Auto-generated method stub

          }

          @Override
          public void windowOpened(WindowEvent e)
          {
            // TODO Auto-generated method stub

          }

        });
        jf.setVisible(true);
        boolean inter = false;
        while (!inter)
        {
          try
          {
            Thread.sleep(10000);
          } catch (Exception e)
          {
            inter = true;
          }
          ;
        }
        jf.dispose();
      }
    }
  }

  public List<Argument> getJobParams()
  {
    List<Argument> argSet = new ArrayList<Argument>();
    // recover options and parameters from GUI
    for (OptionBox opts : optSet.values())
    {
      Option opt = opts.getOptionIfEnabled();
      if (opt != null)
      {
        argSet.add(opt);
      }
    }
    for (ParamBox parambox : paramSet.values())
    {
      Parameter parm = parambox.getParameter();
      if (parm != null)
      {
        argSet.add(parm);
      }
    }

    return argSet;
  }

  String lastParmSet = null;

  Hashtable<String, Object[]> editedParams = new Hashtable<String, Object[]>();

  /**
   * store the given parameters in the user parameter set database.
   * 
   * @param storeSetName
   *          - lastParmSet
   * @param descr
   *          - setDescr.getText()
   * @param jobParams
   *          - getJobParams()
   */
  private void _storeUserPreset(String storeSetName, String descr,
          List<Argument> jobParams)
  {
    // this is a simple hash store.
    Object[] pset;
    editedParams.put(storeSetName, pset = new Object[3]);
    pset[0] = storeSetName;
    pset[1] = descr;
    pset[2] = jobParams;
  }

  private Object[] _getUserPreset(String setName)
  {
    return editedParams.get(setName);
  }

  /**
   * remove the given user preset from the preset stash
   * 
   * @param setName
   */
  private void _deleteUserPreset(String setName)
  {
    editedParams.remove(setName);
  }

  private void syncSetNamesWithStore()
  {
    int n = 0;
    // remove any set names in the drop down menu that aren't either a reserved
    // setting, or a user defined or service preset.
    while (n < setName.getItemCount())
    {
      String item = (String) setName.getItemAt(n);
      if (!isServicePreset(item) && _getUserPreset(item) == null)
      {
        setName.removeItemAt(n);
      }
      else
      {
        n++;
      }

    }
  }

  private void reInitDialog(String nextPreset)
  {
    settingDialog = true;
    syncSetNamesWithStore();
    // updateTable(null,null);
    Object[] pset = _getUserPreset(nextPreset);
    if (pset != null)
    {
      setDescr.setText((String) pset[1]);
      updateTable(null, (List<Argument>) pset[2]);
      lastParmSet = nextPreset;
      validate();
    }
    else
    {
      setDescr.setText("");
      // must be a default preset from service
      Preset p = null;
      try
      {
        PresetManager prman = service.getPresets();
        if (prman != null)
        {
          p = prman.getPresetByName(nextPreset);
        }
      } catch (Exception ex)
      {
        ex.printStackTrace();
      }
      if (p != null)
      {
        updateTable(p, null);
        lastParmSet = nextPreset;
      }
      else
      {
        updateTable(null, null);
      }
    }
    initArgSetModified();
    validate();
    settingDialog = false;

  }

  String curSetName = null;

  @Override
  public void itemStateChanged(ItemEvent e)
  {
    if (settingDialog)
    {
      // ignore event
      return;
    }
    if (e.getSource() == setName)
    {
      String setname = (String) setName.getSelectedItem();
      if (setname == null)
      {
        return;
      }
      if (curSetName == null || !setname.equals(curSetName))
      {
        if (isModified()
                && javax.swing.JOptionPane.showConfirmDialog(this,
                        "Parameter set is modifed - save ?",
                        "Save changes ?",
                        javax.swing.JOptionPane.OK_CANCEL_OPTION) == JOptionPane.OK_OPTION)
        {
          savModified_actionPerformed(null);
        }
        reInitDialog(setname);
      }
    }
  }

  /**
   * last saved name for this user preset
   */
  String lastSetName = null;

  /**
   * last saved value of the description text for this user preset
   */
  String lastDescrText = null;

  @Override
  public void actionPerformed(ActionEvent e)
  {
    if (e.getSource() instanceof Component)
    {
      Component src = (Component) e.getSource();
      if (src.getParent() == setName)
      {
        // rename any existing records we know about for this set.
        String newname = (String) e.getActionCommand();
        String msg=null;
        if (isServicePreset(newname)) {
          JOptionPane.showConfirmDialog(this, "Invalid name - preset already exists.", "Invalid name", JOptionPane.OK_OPTION);
          return;
        }
        curSetName = newname;
        System.err.println("Command " + curSetName + " : "
                + setName.getSelectedItem());
        if (curSetName.trim().equals(setName.getSelectedItem()))
        {
          curSetName = null;
        }
        if (curSetName != null)
        {
          setName.addItem(curSetName);
          setName.setSelectedItem(curSetName);
          argSetModified(setName,
                  lastSetName != null && !curSetName.equals(lastSetName));
          return;
        }

      }
    }
  }

  private void checkDescrModified()
  {
    if (!settingDialog)
    {

      argSetModified(
              setDescr,
              (lastDescrText == null ? setDescr.getText().trim().length() > 0
                      : !setDescr.getText().equals(lastDescrText)));

    }
  }

  @Override
  public void insertUpdate(DocumentEvent e)
  {
    checkDescrModified();
  }

  @Override
  public void removeUpdate(DocumentEvent e)
  {
    checkDescrModified();
  }

  @Override
  public void changedUpdate(DocumentEvent e)
  {
    checkDescrModified();
  }
}